/*
    # ===========================================================================
    # Copyright (C) 2014  Gerhard Hepp
    #
    # This program is free software; you can redistribute it and/or modify it under the terms of
    # the GNU General Public License as published by the Free Software Foundation; either version 2
    # of the License, or (at your option) any later version.
    #
    # This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    # without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    # See the GNU General Public License for more details.
    #
    # You should have received a copy of the GNU General Public License along with this program; if
    # not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    # MA 02110, USA
    # ===========================================================================
*/
#if defined (__AVR_ATmega328P__)
#else
  #define __AVR_ATmega328P__
  #include <avr/iom328p.h>
#endif

#include "Serial.h"

#include <avr/interrupt.h>
#include <stdio.h>



#define MAX_BUFFER 60

uint8_t rx_buffer[MAX_BUFFER];

volatile uint8_t rx_read = 0;
volatile uint8_t rx_write = 0;
int rx_count = 0;

Serial::Serial() {
	rx_read = 0;
	rx_write = 0;
	rx_count = 0;

}

void Serial::init(){

          // __AVR_ATmega328P__

  #define BAUD 19200

          /*Set baud rate */
          unsigned long f = 8000000ul;
          f = f / 16 / BAUD - 1;
          UBRR0H = (unsigned char) (f >> 8);
          UBRR0L = (unsigned char) f;

          /* Enable receiver and transmitter and interrupts*/
          UCSR0B = (1 << RXEN0)
                          | (1 << TXEN0) | (1 << RXCIE0);
          /* Set frame format: 8data, 1stop bit */
          UCSR0C = (3 << UCSZ00);
}

int Serial::available() {
        return (rx_read != rx_write);
}

void Serial::serialOut(const char *s) {
        for (int i = 0; s[i] != 0 ; i++) {

                /* Wait for empty transmit buffer */
                while (!(UCSR0A & (1 << UDRE0)))
                        ;
                UDR0 = s[i];

        }
}

void Serial::serialOutLn(const char *s) {
        serialOut(s);
        serialOut("\n\r");
}

void Serial::serialOut(uint8_t c) {

        /* Wait for empty transmit buffer */
        while (!(UCSR0A & (1 << UDRE0)))
                ;
        UDR0 = c;
}

void Serial::serialOut_P(const char * s) {
        for (int i = 0; 1; i++) {
                char c = (char) pgm_read_byte (s+i);
                if (c == 0)
                        break;
                serialOut(c);
        }

}
void Serial::serialOutLn_P(const char *s) {
        serialOut_P(s);
        serialOut("\n\r");
}

uint8_t Serial::read(void) {
	// if the head isn't ahead of the tail, we don't have any characters
	if (rx_read == rx_write) {
		return -1;
	} else {
		uint8_t c = rx_buffer[rx_read];

		uint8_t next = rx_read + 1;
		if (next >= MAX_BUFFER)
			next = 0;
		rx_read = next;

		return c;
	}
}


ISR(USART_RX_vect) {
	rx_count++;
	uint8_t c = UDR0;

	uint8_t next = rx_write + 1;
	if (next >= MAX_BUFFER)
		next = 0;

	if (next == rx_read) {
		// ignore
	} else {
		rx_buffer[rx_write] = c;
		rx_write = next;
	}
}
