/*
 # ===========================================================================
 # Copyright (C) 2014  Gerhard Hepp
 #
 # This program is free software; you can redistribute it and/or modify it under the terms of
 # the GNU General Public License as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 # without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 # See the GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along with this program; if
 # not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110, USA
 # ===========================================================================
 */

#if defined (DEBUG)
// #define SERIAL
#endif

#if defined (__AVR_ATmega328P__)
#else
#define __AVR_ATmega328P__
#include <avr/iom328p.h>
#endif

#include <stdio.h>

#include <stdint.h>
#include <stdlib.h>
//
#include <avr/interrupt.h>
#include <avr/delay.h>
#include <avr/pgmspace.h>
#include <avr/io.h>

#if defined (DEBUG)
#include "Serial.h"
Serial serial;

#define DEBUG_SERIAL_LN(string) {static const char tx[] PROGMEM = string; serial.serialOutLn_P(tx);}

#endif

//

static const char copyright[] PROGMEM = "copyright Gerhard Hepp 2015";
static const char version[] PROGMEM = "328 2015-03-21 0.0.0";

extern "C"
{
  void
  __cxa_pure_virtual()
  {
    // put error handling here
  }
}

uint8_t old_counter_config[2];
extern volatile uint8_t counter_config[2];

#define HARDWARE_20MHz 20
#define HARDWARE_8MHz   8

#define  DELAY_20MS_DEFAULT (8000000/64/50)
extern volatile uint16_t delay_20ms;


uint8_t hardware = 0;

#define E_STATUSLED_1_0  { PORTD &= ~(1 << 2);}
#define E_STATUSLED_1_1  { PORTD |=  (1 << 2);}

#define E_STATUSLED_2_0  { PORTB &= ~(1 << 1);}
#define E_STATUSLED_2_1  { PORTB |=  (1 << 1);}

void
statusled1(uint8_t value)
{

  if (value)
    E_STATUSLED_1_1
  else
    E_STATUSLED_1_0
}
void
statusled2(uint8_t value)
{
  if (value)
    E_STATUSLED_2_1
  else
    E_STATUSLED_2_0
}

//
// Die Kommandos aus der SPI-Schnittstelle werden in Betriebsarten umgesetzt.
//
// there are four Timer Channels
// For simplicity, these are handled in sequence
//
#define STATE_INITIAL 0

#define STATE_INITIAL_0  1
#define STATE_INITIAL_1  2
// #define STATE_INITIAL_2  3
#define STATE_INITIAL_3  4
#define STATE_MEASURE_0  5
#define STATE_MEASURE_1  6
#define STATE_MEASURE_2  7
#define STATE_MEASURE_3  8

#define STATE_ERROR_0  9
#define STATE_RANGE_ERROR_1  10

#define FALSE (1==0)
#define TRUE ( 1==1)

uint8_t enable_0 = FALSE;
uint8_t enable_1 = FALSE;
uint8_t enable_2 = FALSE;
uint8_t enable_3 = FALSE;
//
uint8_t ultrasonicState_0 = STATE_INITIAL;
uint8_t ultrasonicState_1 = STATE_INITIAL;
uint8_t ultrasonicState_2 = STATE_INITIAL;
uint8_t ultrasonicState_3 = STATE_INITIAL;

uint16_t ultrasonic_0_0 = 0;
uint16_t ultrasonic_1_0 = 0;
uint16_t ultrasonic_2_0 = 0;
uint16_t ultrasonic_3_0 = 0;

uint16_t ultrasonic_0_1 = 0;
uint16_t ultrasonic_1_1 = 0;
uint16_t ultrasonic_2_1 = 0;
uint16_t ultrasonic_3_1 = 0;

uint8_t err_0 = 0;
uint8_t err_1 = 0;
uint8_t err_2 = 0;
uint8_t err_3 = 0;

uint16_t cnt_0 = 0;
uint16_t cnt_1 = 0;
uint16_t cnt_2 = 0;
uint16_t cnt_3 = 0;

extern volatile uint16_t e_sonic_0_0;
extern volatile uint16_t e_sonic_0_1;
extern volatile uint16_t e_cnt_0;
extern volatile uint8_t e_err_0;

extern volatile uint16_t e_sonic_1_0;
extern volatile uint16_t e_sonic_1_1;
extern volatile uint16_t e_cnt_1;
extern volatile uint8_t e_err_1;

extern volatile uint16_t e_sonic_2_0;
extern volatile uint16_t e_sonic_2_1;
extern volatile uint16_t e_cnt_2;
extern volatile uint8_t e_err_2;

extern volatile uint16_t e_sonic_3_0;
extern volatile uint16_t e_sonic_3_1;
extern volatile uint16_t e_cnt_3;
extern volatile uint8_t e_err_3;

uint8_t noiseCanceller = 0;

#define TRIGGER_HIGH_0 (PORTD |= ( 1<<5))
#define TRIGGER_HIGH_1 (PORTD |= ( 1<<6))
#define TRIGGER_HIGH_2 (PORTD |= ( 1<<7))
#define TRIGGER_HIGH_3 (PORTB |= ( 1<<0))

#define TRIGGER_LOW_0 (PORTD &= ~( 1<<5))
#define TRIGGER_LOW_1 (PORTD &= ~( 1<<6))
#define TRIGGER_LOW_2 (PORTD &= ~( 1<<7))
#define TRIGGER_LOW_3 (PORTB &= ~( 1<<0))

#define DATA_0 (PINC & (1 << 5))
#define DATA_1 (PINC & (1 << 4))
#define DATA_2 (PINC & (1 << 3))
#define DATA_3 (PINC & (1 << 2))

uint8_t toggle = 0;
uint8_t roundrobin = 0;
uint8_t next = FALSE;

void
manage_timer()
{

  //
  // check state only on changes
  //
  if ((old_counter_config[1] != counter_config[1])        //
  || (old_counter_config[0] != counter_config[0]))
    {
      if (0)
        {
          if (toggle)
            {
              statusled1(1);
              toggle = 0;
            }
          else
            {
              statusled1(0);
              toggle = 1;
            }
        }

      enable_0 = enable_1 = enable_2 = enable_3 = FALSE;
      noiseCanceller = FALSE;
      next = TRUE;

      if (counter_config[0] & (1 << 0))
        {
          enable_0 = TRUE;
          ultrasonicState_0 = STATE_INITIAL;

        }
      if (counter_config[0] & (1 << 1))
        {
          enable_1 = TRUE;
          ultrasonicState_1 = STATE_INITIAL;

        }
      if (counter_config[0] & (1 << 2))
        {
          enable_2 = TRUE;
          ultrasonicState_2 = STATE_INITIAL;

        }
      if (counter_config[0] & (1 << 3))
        {
          enable_3 = TRUE;
          ultrasonicState_3 = STATE_INITIAL;
        }

      if (counter_config[0] & (1 << 4))
        {
          noiseCanceller = TRUE;
        }
      old_counter_config[1] = counter_config[1];
      old_counter_config[0] = counter_config[0];
    }

  if (next)
    {
      next = FALSE;

      switch (roundrobin)
        {
      case 0:
        if (enable_1)
          {
            roundrobin = 1;
            break;
          }
        if (enable_2)
          {
            roundrobin = 2;
            break;
          }
        if (enable_3)
          {
            roundrobin = 3;
            break;
          }
        if (enable_0)
          {
            roundrobin = 0;
            break;
          }
        roundrobin = 99;
        break;

      case 1:
        if (enable_2)
          {
            roundrobin = 2;
            break;
          }
        if (enable_3)
          {
            roundrobin = 3;
            break;
          }
        if (enable_0)
          {
            roundrobin = 0;
            break;
          }
        if (enable_1)
          {
            roundrobin = 1;
            break;
          }
        roundrobin = 99;
        break;

      case 2:
        if (enable_3)
          {
            roundrobin = 3;
            break;
          }
        if (enable_0)
          {
            roundrobin = 0;
            break;
          }
        if (enable_1)
          {
            roundrobin = 1;
            break;
          }
        if (enable_2)
          {
            roundrobin = 2;
            break;
          }
        roundrobin = 99;
        break;

      case 3:
        if (enable_0)
          {
            roundrobin = 0;
            break;
          }
        if (enable_1)
          {
            roundrobin = 1;
            break;
          }
        if (enable_2)
          {
            roundrobin = 2;
            break;
          }
        if (enable_3)
          {
            roundrobin = 3;
            break;
          }
        roundrobin = 99;
        break;

      case 99:
        if (enable_0)
          {
            roundrobin = 0;
            break;
          }
        if (enable_1)
          {
            roundrobin = 1;
            break;
          }
        if (enable_2)
          {
            roundrobin = 2;
            break;
          }
        if (enable_3)
          {
            roundrobin = 3;
            break;
          }
        roundrobin = 99;
        break;

      default:
        roundrobin = 99;
        return;

        }
      //
    }

  switch (roundrobin)
    {
#include "sonic_snippet.0"
#include "sonic_snippet.1"
#include "sonic_snippet.2"
#include "sonic_snippet.3"
    }

}

int
main()
{
  delay_20ms = DELAY_20MS_DEFAULT;
  hardware = HARDWARE_8MHz;

  counter_config[0] = 0x00;
  counter_config[1] = 0x00;   //

// when timer bits are changing, then manage them.
// set them to 0, so manage_code is starting counting.
  old_counter_config[0] = 0;
  old_counter_config[1] = 0;

//  status = 0;

// -----------------------------------------------------
// PORTB
//    PB.4 ist MISO
//    PB.0 ist TRIGGER 3
//    PB.1 ist LED
  DDRB = 0b00010011;
  PORTB = 0b11111111;   // enable pull up for not needed inputs

// -----------------------------------------------------
// PORT C

  DDRC = 0b00000000;
  PORTC = 0b11111111;   // enable pull up for not needed inputs

// -----------------------------------------------------
// PORTD

// Pin 7 ist LED-Ausgang
// Pin 4 ist Signal Ausgang (pullup enabled)
//
// die LED sind  gegen Masse angeschlossen
//
  DDRD = 0b11110100;
  PORTD = 0b11111111;

// -------------------------------------------------
// enable pin change interrupt, used to reset the SPI logic
//
  PCICR |= (1 << PCIE0);
  PCMSK0 |= (1 << PCINT2);

// -------------------------------------------------
// no power savings

  PRR = 0;

// -------------------------------------------------
// enable SPI
//
  SPCR = (1 << SPIE) | (1 << SPE);

//
  sei();

#ifdef SERIAL
  serial.init();
  DEBUG_SERIAL_LN(version);
#endif

  for (uint8_t i = 0; i < hardware; i++)
    {
      statusled1(1);
      statusled2(0);

      _delay_ms(80);

      statusled1(0);
      statusled2(1);

      _delay_ms(80);
    }

  while (1)
    {
      manage_timer();

    }

  return 0;

}
