# -*- coding: utf-8 -*-
    # ===========================================================================
    # Copyright (C) 2014  Gerhard Hepp
    #
    # This program is free software; you can redistribute it and/or modify it under the terms of
    # the GNU General Public License as published by the Free Software Foundation; either version 2
    # of the License, or (at your option) any later version.
    #
    # This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    # without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    # See the GNU General Public License for more details.
    #
    # You should have received a copy of the GNU General Public License along with this program; if
    # not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    # MA 02110, USA
    # ===========================================================================

 
import time
import RPi.GPIO as GPIO
import programmer.Programmer

import spidev

            
class Peripherie:
    debug = False

    spi_bus = 0
    spi_device = 0
    # GPIO Pin Numbers (BMC-Nummerierung)

    RESET = 24
    LED = 17
    
    # Kommandos für den Prozessor Atmega 238
    GET_VERSION = 0x6e
    GET_TESTDATA = 0x6f
    
    GET_DELAY20MS = 0x6a
    SET_DELAY20MS = 0x6b
    
    SET_COUNTER_CONFIG = 0x65
    GET_COUNTER_CONFIG = 0x66
    GET_COUNTER_0      = 0x60
    GET_COUNTER_1      = 0x61
    GET_COUNTER_2      = 0x62
    GET_COUNTER_3      = 0x63
    
    SET_LED_0_0 = 0x64
    SET_LED_0_1 = 0x68

    SET_LED_1_0 = 0x67
    SET_LED_1_1 = 0x69
    
    
    def __init__(self, bus=0, device=0, spi_speed=240000):
        
        GPIO.setmode(GPIO.BCM)
        GPIO.setwarnings(False)
        
        GPIO.setup(self.RESET, GPIO.OUT)
        GPIO.output(self.RESET, GPIO.HIGH)

        GPIO.setup(self.LED, GPIO.OUT)
        GPIO.output(self.LED, GPIO.HIGH)
        
        self.spi = spidev.SpiDev()
        self.spi.open(self.spi_bus, self.spi_device)
            
        self.spi.max_speed_hz = spi_speed
        
    def close(self):
        self.spi.close()
        GPIO.output(self.LED, GPIO.LOW)

    def rst(self, value):
        GPIO.output(self.RESET, not(value) )
        
        
    def irc_signal(self):
        return GPIO.input(self.IRC_SIGNAL)
    
    #
    # CPU-command
    #
    # rückgabe wert 0xff: erfolgreich, Platz im Puffer
    # rückgabe wert 0xfe: erfolgreich, kein Platz mehr im Puffer
    # rückgabe wert 0xfd: nicht erfolgreich, da kein Platz im Puffer

    def cpu_cmd_r(self, l):
        if self.debug:
            print("cpu_cmd_r: ")
            for i in range(len(l)):
                print("{0:02x} ".format(l[i]))
        # speed so langsam setzen, damit interrupt genuegend zeit hat
                
        r = self.spi.xfer2(list(l))
        return r

    def cpu_cmd(self, l):
        
        # speed so langsam setzen, damit interrupt genuegend zeit hat
        
        r = self.spi.xfer2(list(l))
    
    #
    #
    # ############################# PROGRAMMIERUNG
    #                
    #
    def program_reset_on(self):
        """callback method for programming"""
        self.rst(True)
    
    def program_reset_off(self):
        """callback method for programming"""
        self.rst(False)

    def program_spi_xfer(self, data):
        """callback method for programming"""
        self.spi.max_speed_hz = 200000
        r = self.spi.xfer2(data)
        return r
    
    def program_init(self):
        """special initialization code (e.g. switch off other hardware) """
        self.program_reset_on()
        pass

    def program_close(self):
        self.program_reset_off()

    def readCode(self, fileName):
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_readCode(fileName)
        self.program_close()

    def readFuses(self):
        print("read processor fuses")
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_readFuses()
        self.program_close()

    def writeFuse(self, value):
        """write fuse 'low fuse'"""
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_writeFuse(value)
        self.program_close()

    def  writeCode(self, fileName):
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_writeCode(fileName)
        self.program_close()

    def clearCode(self):
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_clearCode()
        self.program_close()
       
    # ################################################
    
    def cpu_led_0_on(self):
        if self.debug:
            print("cpu_led_on")
        self.cpu_cmd(list([self.SET_LED_0_1]))
        
    def cpu_led_0_off(self):
        if self.debug:
            print("cpu_led_off")
        self.cpu_cmd(list([self.SET_LED_0_0]))

    def cpu_led_1_on(self):
        if self.debug:
            print("cpu_led_on")
        self.cpu_cmd(list([self.SET_LED_1_1]))
        
    def cpu_led_1_off(self):
        if self.debug:
            print("cpu_led_off")
        self.cpu_cmd(list([self.SET_LED_1_0]))
        
    def cpu_get_version(self):
        if self.debug:
            print("cpu_get_version")
        
        r = self.cpu_cmd_r(list([self.GET_VERSION, 0, 0]))
        return r

    def cpu_get_delay(self):
        if self.debug:
            print("cpu_get_delay")
        
        r = self.cpu_cmd_r(list([self.GET_DELAY20MS, 0, 0]))
        return r

    def cpu_set_delay(self, value):
        if self.debug:
            print("cpu_set_delay", value)
        
        self.cpu_cmd_r(list([self.SET_DELAY20MS, value & 0xff, (value & 0xff00 )/256 ]))

    def cpu_get_counter_0(self):
        if self.debug:
            print("cpu_get_counter 0")
        
        r = self.cpu_cmd_r(list([self.GET_COUNTER_0, 0, 0, 0, 0, 0, 0, 0]))
        return r
    
    def cpu_get_counter_1(self):
        if self.debug:
            print("cpu_get_counter 1")
        
        r = self.cpu_cmd_r(list([self.GET_COUNTER_1, 0, 0, 0, 0, 0, 0, 0]))
        return r
    
    def cpu_get_counter_2(self):
        if self.debug:
            print("cpu_get_counter 1")
        
        r = self.cpu_cmd_r(list([self.GET_COUNTER_2, 0, 0, 0, 0, 0, 0, 0]))
        return r
    
    def cpu_get_counter_3(self):
        if self.debug:
            print("cpu_get_counter 1")
        
        r = self.cpu_cmd_r(list([self.GET_COUNTER_3, 0, 0, 0, 0, 0, 0, 0]))
        return r
    
    def cpu_get_testdata(self):
        if self.debug:
            print("cpu_get_testdata")
        
        r = self.cpu_cmd_r(list([ self.GET_TESTDATA, 0, 0, 0, 0, 0, 0, 0]))
        return r
    
    
    def cpu_set_counter_config(self, c0, c1):
        if self.debug:
            print("cpu_set_counterconfig", c0, c1)
        
        r = self.cpu_cmd_r( list( [self.SET_COUNTER_CONFIG, c0, c1]))
        return r


    def cpu_get_counter_config(self):
        if self.debug:
            print("cpu_get_counterconfig")
        
        r = self.cpu_cmd_r(list([self.GET_COUNTER_CONFIG, 0, 0 ]))
        return r

